# POPFILE LOADABLE MODULE
package Proxy::MERC;

use Proxy::Proxy;
@ISA = ("Proxy::Proxy");

# ---------------------------------------------------------------------------------------------
#
# This module handles communications with Mercury/32 for POPFile.
my $version =  "1.22.0";
#
# Copyright (c) 2001-2003 John Graham-Cumming
#
#   This file is part of POPFile
#
#   POPFile is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   POPFile is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with POPFile; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#   Modified by     Eduardo Subelman(subelman@markmatrix.com)
#
# ---------------------------------------------------------------------------------------------

use strict;
use warnings;
use locale;

# A handy variable containing the value of an EOL for networks
my $eol = "\015\012";

#----------------------------------------------------------------------------
# new
#
#   Class new() function
#----------------------------------------------------------------------------
sub new
{
    my $type = shift;
    my $self = Proxy::Proxy->new();

    # Must call bless before attempting to call any methods

    bless $self, $type;

    $self->name( 'merc' );

    $self->{child_} = \&child__;
    $self->{connection_timeout_error_} = '-ERR';
    $self->{connection_failed_error_}  = '-ERR could not open file';
    $self->{good_response_}            = '^\+OK';

    return $self;
}

# ---------------------------------------------------------------------------------------------
#
# initialize
#
# Called to initialize the MERC proxy module
#
# ---------------------------------------------------------------------------------------------
sub initialize
{
    my ( $self ) = @_;

    # Enabled by default
    $self->config_( 'enabled', 1);

    # By default we don't fork on Windows
    $self->config_( 'force_fork', ($^O eq 'MSWin32')?0:1 );

    # Default ports for service and the user interface
    # use sunrpc port because it is not used in Windows(?)
    $self->config_( 'port', 111 );

    # Only accept connections from the local machine
    $self->config_( 'local', 1 );

    $self->log_( 1, 'Initialized merc ' . $version );

    return $self->SUPER::initialize();
}

# ---------------------------------------------------------------------------------------------
#
# start
#
# ---------------------------------------------------------------------------------------------
sub start
{
    my ( $self ) = @_;

    # If we are not enabled then no further work happens in this module

    if ( $self->config_( 'enabled' ) == 0 ) {
        return 2;
    }

    # Tell the user interface module that we having a configuration
    # item that needs a UI component

    $self->register_configuration_item_( 'configuration',              # PROFILE BLOCK START
                                         'merc_fork_and_port',
                                         'merc-configuration.thtml',
                                         $self );                      # PROFILE BLOCK STOP

    $self->register_configuration_item_( 'security',                   # PROFILE BLOCK START
                                         'merc_local',
                                         'merc-security-local.thtml',
                                         $self );                      # PROFILE BLOCK STOP

    $self->log_( 1, 'Started merc ' . $version );

    return $self->SUPER::start();
}

# ---------------------------------------------------------------------------------------------
#
# child__
#
# The worker method that is called when we get a good connection from a client
#
# $client         - an open stream to a MERC client
# $session        - API session key
#
# ---------------------------------------------------------------------------------------------
sub child__
{
    my ( $self, $client, $session ) = @_;

    # Tell the client that we are ready for commands and identify our version number

    $self->tee_( $client, "+OK " . "$eol" );

    # Retrieve commands from the client and process them until the client disconnects or
    # we get a specific QUIT command

    while  ( <$client> ) {
        my $inputfilename;

        $inputfilename = $_;

        # Clean up the inputfilename so that it has a nice clean $eol at the end

        $inputfilename =~ s/(\015|\012)//g;

        $self->log_( 2, "Command: --$inputfilename--" );

        # we now have a file name
        if (open(RETRFILE, "<$inputfilename")) {
            my $class;

            my $slot;
            ( $class, $slot ) = $self->{classifier__}->classify_and_modify( $session,  \*RETRFILE, \*NULLFILE, 0, '', 0, 0, $eol );

            close RETRFILE;
            # Add the XTC header
            my $xtc = '';
            if ($self->{classifier__}->get_bucket_parameter( $session, $class, 'xtc' )) {
                $xtc = "$class";
            }
            # Add the Subject modifier
            my $sub = '';
            if ($self->{classifier__}->get_bucket_parameter( $session, $class, 'subject' )) {
                $sub = $self->{classifier__}->config_('subject_mod_left') . "$class" .
                      $self->{classifier__}->config_('subject_mod_right');
            }
            # Add the XPL header
            my $xpl = '';
            if ($self->{classifier__}->get_bucket_parameter( $session, $class, 'xpl' )) {
                $xpl .= "http://";
                $xpl .= $self->module_config_( 'html', 'local' )?"127.0.0.1":$self->module_config_( 'bayes', 'hostname' );
                $xpl .= ":" . $self->module_config_( 'html', 'port' ) . "/jump_to_message?view=$slot";
            }

            # Tell the client about it
            $self->tee_( $client, "+OK " . "$xtc" . "+" . "$sub" . "+" . $xpl . "$eol" );

            close $client;

            $self->log_( 2, "MERC forked child done" );
            last;
        }
        else {
            $self->tee_( $client, "-ERR " . "$eol" );
        }
    }
}

# ---------------------------------------------------------------------------------------------
#
# configure_item
#
#    $name            The name of the item being configured, was passed in by the call
#                     to register_configuration_item
#    $templ           The loaded template that was passed as a parameter
#                     when registering
#    $language        Reference to the hash holding the current language
#
# ---------------------------------------------------------------------------------------------

sub configure_item
{
    my ( $self, $name, $templ, $language ) = @_;

    if ( $name eq 'merc_fork_and_port' ) {
        $templ->param( 'merc_port' => $self->config_( 'port' ) );
        $templ->param( 'merc_force_fork_on' => $self->config_( 'force_fork' ) );
    }

    if ( $name eq 'merc_local' ) {
        $templ->param( 'merc_local_on' => $self->config_( 'local' ) );
     }

    #$self->SUPER::configure_item( $name, $templ, $language );
}

# ---------------------------------------------------------------------------------------------
#
# validate_item
#
#    $name            The name of the item being configured, was passed in by the call
#                     to register_configuration_item
#    $templ           The loaded template
#    $language        Reference to the hash holding the current language
#    $form            Hash containing all form items
#
# ---------------------------------------------------------------------------------------------

sub validate_item
{
    my ( $self, $name, $templ, $language, $form ) = @_;

    if ( $name eq 'merc_fork_and_port' ) {

        if ( defined($$form{merc_force_fork}) ) {
            $self->config_( 'force_fork', $$form{merc_force_fork} );
        }

        if ( defined($$form{merc_port}) ) {
            if ( ( $$form{merc_port} >= 1 ) && ( $$form{merc_port} < 65536 ) ) {
                $self->config_( 'port', $$form{merc_port} );
                $templ->param( 'merc_port_feedback' => sprintf( $$language{Configuration_MERCUpdate}, $self->config_( 'port' ) ) );
             } else {
                $templ->param( 'merc_port_feedback' => "<div class=\"error01\">$$language{Configuration_Error3}</div>" );
             }
        }
    }

    if ( $name eq 'merc_local' ) {
        if ( defined $$form{merc_local} ) {
            $self->config_( 'local', $$form{merc_local} );
        }
    }


    #$self->SUPER::validate_item( $name, $templ, $language, $form );
}

# required et al.
